#include "arena.h"
#include <rg3DRom.h>
#include <rgOpenGL.h>

phArena::phArena()
{
	m_mdl = 0;
	dsplst_ice_surface = 0;
	dsplst_boards = 0;
}

phArena::~phArena()
{
	clear();
}

void phArena::clear()
{
	if( m_mdl) { delete m_mdl; m_mdl = 0;}
	clear_display_list( dsplst_ice_surface);
	clear_display_list( dsplst_boards);
}

int phArena::load( const char *fname)
{
	clear();
	m_mdl = new sk3DRomModel();
	const int err = m_mdl->load( fname);
	if( err) return err;
	const char *paths[] = { ".", "./media", "../media/bmp"};
	m_mdl->load_textures( paths, 3);
	return 0;
}

void phArena::prepare_for_rendering()
{
	if( !m_mdl) return;

	m_mdl->transform_skel_frame();
	
	compile_face_list( "Ice Surface", dsplst_ice_surface);

	{ // Prepare the boards for rendering
		const char *tmp[] = { "Dasher Board Top", "Boards",
			"Top of boards (red part)"};
		compile_face_list( tmp, 3, dsplst_boards);
	}

}

void phArena::compile_face_list( const char *p_name, GLuint &id)
{
	const char *tmp[] = { p_name};
	compile_face_list( tmp, 1, id);
}

void phArena::compile_face_list( const char *p_name[], const int p_count,
	GLuint &id)
{
	if( id) { glDeleteLists( id, 1); id = 0;}
	id = glGenLists( 1);
	glNewList( id, GL_COMPILE);
	for( int i = 0; i < p_count; i++) {
		int count;
		const int *flist = m_mdl->get_faces_in_group(
			m_mdl->get_group_id( p_name[i]), count);
		if( flist) m_mdl->render_faces( true, true, flist, count);
	}
	glEndList();
}

void phArena::clear_display_list( GLuint &id)
{
	if( 0 == id) return;
	glDeleteLists( id, 1);
	id = 0;
}

void phArena::draw_boards() const
{
	if( dsplst_boards) glCallList( dsplst_boards);
}

void phArena::draw_glass() const
{
}
