#include "main_header.h"
#include "engine_game.h"
#include "engine_graphics.h"
#include <rgTime.h>
#include <rgNetwork.h>
#include <iostream>
#include <rg3DRom.h>
#include "things/things.h"

GameEngine::GameEngine( MainData *p_mdata)
{
	mdata = p_mdata;
	
	// Init network engine
	m_console = new rgSocket();

	// Init timing engine
	m_game_clock_hz = 60;
	m_game_clock_tick = 1.0 / double( m_game_clock_hz);
	m_game_timer = new rgTimer( m_game_clock_tick);
	m_game_clock = 0;

	// Init graphics
	gfx_engine = new GraphicsEngine( this);

	// Init 3D Models
	m_player_model = 0;

	// Init thing manager
	m_thing_manager = 0;
}

GameEngine::~GameEngine()
{
	clear();
}

void GameEngine::clear()
{
	if( m_game_timer) { delete m_game_timer; m_game_timer = 0;}
	if( m_console) { delete m_console; m_console = 0;}
	if( gfx_engine) { delete gfx_engine; gfx_engine = 0;}

	if( m_player_model) { delete m_player_model; m_player_model = 0;}

	if( m_thing_manager) { delete m_thing_manager; m_thing_manager = 0;}
}

int GameEngine::load_arena( const char *fname)
{
	if( !gfx_engine) return -1;
	return gfx_engine->load_arena( fname);
}

void GameEngine::process()
{
	long long clk = game_clocks_pending();
	while( clk--) {
		m_thing_manager->process();

		m_game_clock++;
		if( !(game_clock() % 1000))
			std::cerr << "game second passed. " << game_clock()
				<< "\n";
	}

	gfx_engine->process();
}

long long GameEngine::game_clocks_pending()
{
	if( m_game_timer->time_unit_passed())
		return (long long)(m_game_timer->elapsed_mili_sec());
	else return 0;
}

int GameEngine::load_startup_file( const char *fname)
{
	return 0;
}

int GameEngine::load_player_model( const char *fname)
{
	if( !fname) return -1;
	if( m_player_model) delete m_player_model;
	m_player_model = new sk3DRomModel();
	if( !m_player_model) return -2;
	m_player_model->load( fname);
	return 0;
}

int GameEngine::create_thing_manager()
{
	if( m_thing_manager) return 0;
	if( !(m_thing_manager = new ThingManager( this))) return -1;
	return 0;
}
