#include "things.h"
#include "../engine_game.h"

void phDebugger::process( ThingManager *tm)
{
	std::cout << "process debugger\n";
}

ThingManager::ThingManager( GameEngine *p_game_engine)
{
	m_game_engine = p_game_engine;

	m_max_things = TM_MAX_THINGS;
	m_things = new ThingControlBlock[ m_max_things];
        init_things();
}

ThingManager::~ThingManager()
{
	clear();
}

void ThingManager::clear()
{
	// FIXME: Remove thing from thing manager.
	//while( !(m_thing.empty())) {
		//if( (m_thing[0]).m_thing) delete (m_thing[0]);
		//m_thing.erase( m_thing.begin());
	//}
}

int ThingManager::create_thing( phThing *thg)
{
	if( !thg) return -1;
	if( m_thing_free < 0) return -2;
	const int tcb_index = m_thing_free;
	ThingControlBlock &tcb = m_things[ m_thing_free];
	m_thing_count++;

	// Set the new head free block.
	m_thing_free = tcb.m_next;
	if( m_thing_free >= 0) {
		ThingControlBlock &fcb = m_things[ m_thing_free];
		fcb.m_prev = -1;
	}

	// The new tcb will be the at the head of the "things" list.
	tcb.m_next = m_thing_head;
	m_thing_head = tcb_index;
	tcb.m_prev = -1;
	tcb.m_thing = thg;
	move_thing_to_proc( tcb, m_proc[10]);
	//m_thing.push_back( ThingControlBlock( thg, m_proc[10]));
	return 0;
}

void ThingManager::move_thing_to_proc( ThingControlBlock &tcb,
	std::list <ThingControlBlock *> &p_proc)
{
	if( tcb.m_proc == (&p_proc)) return;	// Not neccasary, but efficient
	remove_thing_from_proc( tcb);
	tcb.m_proc = &p_proc;
	if( !p_proc) return;
	p_proc.push_back( &tcb);
}

void ThingManager::remove_thing_from_proc( ThingControlBlock &tcb)
{
	if( !(tcb.m_proc)) return;
	tcb.m_proc->erase( tcb.m_iter);
	tcb.m_proc = 0;
}

void ThingManager::process()
{
	process_list( m_proc[0]);
	if( !((ge())->game_clock() & 0x1)) process_list( m_proc[1]);
	if( !((ge())->game_clock() & 0x3)) process_list( m_proc[2]);
	if( !((ge())->game_clock() & 0x7)) process_list( m_proc[3]);
	if( !((ge())->game_clock() & 0xF)) process_list( m_proc[4]);
	if( !((ge())->game_clock() & 0x1F)) process_list( m_proc[5]);
	if( !((ge())->game_clock() & 0x3F)) process_list( m_proc[6]);
	if( !((ge())->game_clock() & 0x7F)) process_list( m_proc[7]);
	if( !((ge())->game_clock() & 0xFF)) process_list( m_proc[8]);
	if( !((ge())->game_clock() & 0x1FF)) process_list( m_proc[9]);
	if( !((ge())->game_clock() & 0x3FF)) process_list( m_proc[10]);
	if( !((ge())->game_clock() & 0x7FF)) process_list( m_proc[11]);
	if( !((ge())->game_clock() & 0xFFF)) process_list( m_proc[12]);
	if( !((ge())->game_clock() & 0x1FFF)) process_list( m_proc[13]);
	if( !((ge())->game_clock() & 0x3FFF)) process_list( m_proc[14]);
	if( !((ge())->game_clock() & 0x7FFF)) process_list( m_proc[15]);
}

void ThingManager::process_list( std::list <ThingControlBlock *> &p_proc)
{
	int count = int( p_proc.size());
	ThingControlBlock * &p = p_proc.front();
	while( count--) {
		if( p->m_thing) p->m_thing->process( this);
		p++;
	}
}

void ThingManager::init_things()
{
	m_thing_count = 0;
	m_thing_head = -1;
	m_thing_free = 0;
	for( int i = 0; i < m_max_things; i++) {
		m_things[i].m_prev = i - 1;
		m_things[i].m_next = i + 1;
	}
	m_things[ m_max_things - 1].m_next = -1;
}
